<?php
/**
 * @package	    UpdateSiteFix
 * @copyright	Copyright (c)2016 Nicholas K. Dionysopoulos / AkeebaBackup.com
 * @license	    GNU General Public License version 3 or later
 */

defined('_JEXEC') or die;

/**
 * Abstract update site fixer
 */
abstract class USFAbstract
{
	/**
	 * Return an array of detected update site definitions
	 *
	 * @return  array
	 */
	abstract public function scan();

	/**
	 * Parse an XML manifest and extract the update sites
	 *
	 * @param   string           $filePath            The full site path to the XML manifest
	 * @param   string|callable  $element             The extension element for this manifest, or a callable(SimpleXMLElement $xml)
	 * @param   string           $extensionType       Extension type, i.e. component, package, plugin, module, file, ...
	 * @param   int              $client              Client ID, 1 = admin , 0 = site (only for modules and templates)
	 * @param   string           $folder              Extension folder (only for plugins)
	 * @param   int              $defaultExtensionId  Default extension ID in case I can't find it. Leave 0 to skip the manifest in this case.
	 *
	 * @return  array
	 */
	protected function parseManifest($filePath, $element, $extensionType, $client = null, $folder = null, $defaultExtensionId = 0)
	{
		$ret = array();

		$data = @file_get_contents($filePath);

		if (empty($data))
		{
			return $ret;
		}

		try
		{
			$xml = new SimpleXMLElement($data);
		}
		catch (\Exception $e)
		{
			return $ret;
		}

		$tagName = $xml->getName();

		if (!in_array($tagName, array('extension', 'component')))
		{
			return $ret;
		}

		/** @var SimpleXMLElement $child */
		foreach ($xml->children() as $child)
		{
			$tagName = $child->getName();

			if (!in_array($tagName, array('updatesites', 'updateservers')))
			{
				continue;
			}

			/** @var SimpleXMLElement $siteNode */
			foreach ($child->children() as $siteNode)
			{
				$tagName = $siteNode->getName();

				if ($tagName != 'server')
				{
					continue;
				}

				$realElement = $element;

				if (is_callable($element))
				{
					$realElement = call_user_func($element, $xml);
				}

				// type, priority, name, [location]
				$info = array(
					'eid'      => 0,
					'type'     => 'extension',
					'priority' => '1',
					'name'     => 'Update site for ' . $realElement . ' ' . $extensionType,
					'location' => '',
				);

				$info['eid'] = $this->getExtensionId($realElement, $extensionType, $client, $folder);

				if (empty($info['eid']))
				{
					$info['eid'] = $defaultExtensionId;
				}

				if (empty($info['eid']))
				{
					continue;
				}

				$info['location'] = (string) $siteNode;
				$info['location'] = trim($info['location']);

				foreach ($siteNode->attributes() as $k => $v)
				{
					$k = (string) $k;
					$v = (string) $v;

					switch ($k)
					{
						case 'type':
						case 'priority':
						case 'name':
							$info[$k] = $v;
							break;
					}
				}

				$ret[] = $info;
			}
		}

		return $ret;
	}

	/**
	 * Gets the ID of an extension
	 *
	 * @param   string  $element  Extension element, e.g. com_foo, mod_foo, lib_foo, pkg_foo or foo (CAUTION: plugin, file!)
	 * @param   string  $type     Extension type: component, module, library, package, plugin or file
	 * @param   int     $client   Modules: 1 (admin) / 0 (site).
	 * @param   null    $folder   Plugins: plugin folder.
	 *
	 * @return  int  Extension ID or 0 on failure
	 */
	protected function getExtensionId($element, $type = 'component', $client = null, $folder = null)
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
		            ->select($db->qn('extension_id'))
		            ->from($db->qn('#__extensions'))
		            ->where($db->qn('element') . ' = ' . $db->q($element))
		            ->where($db->qn('type') . ' = ' . $db->q($type));

		// Plugin? We should look for a folder
		if ($type == 'plugin')
		{
			$folder = empty($folder) ? 'system' : $folder;

			$query->where($db->qn('folder') . ' = ' . $db->q($folder));
		}

		// Module?
		if ($type == 'module')
		{
			$client = empty($client) ? 0 : $client;

			$query->where($db->qn('client_id') . ' = ' . $db->q($client));
		}

		// Template?
		if ($type == 'template')
		{
			$client = empty($client) ? 0 : $client;

			$query->where($db->qn('client_id') . ' = ' . $db->q($client));
		}

		try
		{
			$id = $db->setQuery($query, 0, 1)->loadResult();
		}
		catch (Exception $e)
		{
			$id = 0;
		}

		return empty($id) ? 0 : (int) $id;
	}
}