<?php
/**
 * @package	    UpdateSiteFix
 * @copyright	Copyright (c)2016 Nicholas K. Dionysopoulos / AkeebaBackup.com
 * @license	    GNU General Public License version 3 or later
 */

defined('_JEXEC') or die;

final class UpdateSiteFixer
{
	/**
	 * Discovers all possible update sites.
	 *
	 * Each element of the return array is an array itself:
	 * eid          Extension ID
	 * type         Update site type
	 * priority     Update site priority
	 * name         Update site name
	 * location     URL
	 *
	 * @return  array
	 */
	public function discoverUpdateSites()
	{
		require_once __DIR__ . '/abstract.php';

		$updateSites = array();

		JLoader::import('joomla.filesystem.folder');
		$handlers = JFolder::files(__DIR__ . '/scanners', '\.php$', false, true);
		foreach ($handlers as $filename)
		{
			require_once $filename;
			$className = basename($filename, '.php');
			$className = 'USF' . ucfirst($className);

			if (!class_exists($className))
			{
				continue;
			}

			/** @var USFAbstract $o */
			$o = new $className;
			$moreUpdateSites = $o->scan();

			if (is_array($moreUpdateSites) && count($moreUpdateSites))
			{
				$updateSites = array_merge($updateSites, $moreUpdateSites);
			}
		}

		return $updateSites;
	}

	public function applyUpdateSites($updateSites)
	{
		$count = 0;

		if (!is_array($updateSites))
		{
			return $count;
		}

		if (empty($updateSites))
		{
			return $count;
		}

		foreach ($updateSites as $site)
		{
			$oldSite = $this->getUpdateSite($site['type'], $site['location']);

			if (is_array($oldSite) && isset($oldSite['update_site_id']) && !empty($oldSite['update_site_id']))
			{
				continue;
			}

			if ($this->makeUpdateSite($site))
			{
				$count++;
			}
		}

		return $count;
	}

	public function getUpdateSite($type, $location)
	{
		$db = JFactory::getDbo();

		$query = $db->getQuery(true)
			->select('*')
			->from($db->qn('#__update_sites'))
			->where($db->qn('type') . ' = ' . $db->q($type))
			->where($db->qn('location') . ' = ' . $db->q($location));
		try
		{
			$ret = $db->setQuery($query)->loadAssoc();

			if (empty($ret))
			{
				return null;
			}
		}
		catch (Exception $e)
		{
			return null;
		}

		return $ret;
	}

	public function getUpdateSiteExtension($updateSiteId)
	{
		$db = JFactory::getDbo();

		$query = $db->getQuery(true)
		            ->select($db->qn('extension_id'))
		            ->from($db->qn('#__update_sites_extensions'))
		            ->where($db->qn('update_site_id') . ' = ' . $db->q($updateSiteId));
		try
		{
			$ret = $db->setQuery($query)->loadResult();

			if (empty($ret))
			{
				return null;
			}
		}
		catch (Exception $e)
		{
			return null;
		}

		return $ret;
	}

	public function makeUpdateSite($siteDefinition)
	{
		$updateSite = array(
			'name'                 => $siteDefinition['name'],
			'type'                 => $siteDefinition['type'],
			'location'             => $siteDefinition['location'],
			'enabled'              => 1,
			'last_check_timestamp' => 0,
			'extra_query'          => null,
		);

		$updateSite = (object) $updateSite;

		$db = JFactory::getDbo();

		try
		{
			if (!$db->insertObject('#__update_sites', $updateSite))
			{
				return false;
			}

			$updateSiteId = $db->insertid();
		}
		catch (Exception $e)
		{
			return false;
		}

		$o = (object) array(
			'update_site_id' => $updateSiteId,
			'extension_id'  => $siteDefinition['eid']
		);

		try
		{
			if (!$db->insertObject('#__update_sites_extensions', $o))
			{
				throw new Exception('Failed to create update site');
			}

			$ret = true;
		}
		catch (Exception $e)
		{
			$ret = false;
		}

		if ($ret === false)
		{
			$query = $db->getQuery(true)
			            ->delete($db->qn('#__update_sites'))
			            ->where($db->qn('update_site_id') . ' = ' . $db->q($updateSiteId));

			try
			{
				$db->setQuery($query)->execute();
			}
			catch (Exception $e)
			{
				return false;
			}
		}

		return true;
	}
}